function craftSetup()    
    --make entity and scene
    scene = craft.scene()
    scene.sun.rotation=quat.eulerAngles(30,45,30)
    v = {} --placeholder allowing me to comment out orbit viewer
    v=scene.camera:add(OrbitViewer, vec3(2.5, 3.0, 0.0), 14.0, 0, 1000)
 --   v=scene.camera:add(OrbitViewer, vec3(0, 0, 0.0), 104.0, 0, 1000)
    v.rx=-35
    v.ry=0
    scene.sky.material.sky = color(92, 215, 228)       
    scene.sky.material.horizon = color(0, 26, 255)       
    scene.sky.material.ground = color(125, 0, 255)   
    entity = scene:entity()
    entity.model = craft.model()
    entity.material = makeNewShader()
    makeCraftSquares()
end

function makeCraftSquares()
    entity.model.positions = {}
    local overallSideSize = 5
    local positions, colors, indices = vertsAndColorsBasedOn(squaresPerSide, overallSideSize)
    entity.model.positions = positions
    entity.model.colors = colors
    entity.model.indices = indices
    local numVertices = #positions -- Or however many vertices your model has
    entity.material.numVertices = numVertices
    entity.material.showDebugColors = false;
    entity.material.verticalDistanceIncrement = verticalDistanceIncrement
    -- Encode and set the distances texture   
    entity.model.uvs = makeUVArrayEncodingIndices(numVertices)
    local distances = distancesBySquareBelongedTo(squaresPerSide, verticalDistanceIncrement)
    if not hasMadeMaterial then        
        entity.material.distancesTexture = encodeFloatTableAsImage(distances)
        hasMadeMaterial = true
    end
end
    
function makeUVArrayEncodingIndices(size)
    local uvs = {}
    local bunchSize = 1
    -- Iterate over the distances, applying each one to six vertices
    for i = 0, size - 1, bunchSize  do
        local encodedIndex = i / 10000
        -- Set the same UV for the six vertices of the current square
        for j = 1, bunchSize do
            table.insert(uvs, vec2(encodedIndex, 0))
        end
    end
    return uvs
end

function encodeFloatTableAsImage(floats)
    print("making")
    local imgWidth = #floats
    local img = image(imgWidth, 1)
    for i, value in ipairs(floats) do
        local r,g,b,a = encodeFloat(value)
        img:set(i, 1, r, g, b, a) -- Alpha set to 255 (fully opaque)
    end   
    return img
end

function encodeFloat(floatValue)
    local adjustmentFactor = 0
    local signIndicator = floatValue >= 0 and 100 or 200 -- 100 for positive, 200 for negative
    
    -- Ensure the value is positive for simplicity in handling
    adjustedValue = math.abs(floatValue)
    
    -- Adjust the float to have at most two digits before the decimal
    while adjustedValue >= 100 do
        adjustedValue = adjustedValue / 10
        adjustmentFactor = adjustmentFactor + 1
    end
    while adjustedValue < 10 and adjustmentFactor > 0 do
        adjustedValue = adjustedValue * 10
        adjustmentFactor = adjustmentFactor - 1
    end
    
    local alphaValue = signIndicator + adjustmentFactor
    
    -- Apply the original encoding method
    local intPart = math.floor(adjustedValue)
    local firstDecimalPart = math.floor((adjustedValue - intPart) * 255)
    local secondDecimalPart = math.floor(((adjustedValue - intPart) * 255 - firstDecimalPart) * 255)
    
    return intPart, firstDecimalPart, secondDecimalPart, alphaValue
end

function makeNewShader()
    if not shaderCount then
        shaderCount = 0
    end
    shaderCount = shaderCount + 1
    local newShaderName = "Undulating Shader"..shaderCount
    local newShader = {
        name = newShaderName,
        
        options =
        {
            USE_COLOR = { true },
        },
        
        properties =
        {
            time = {"float", 0.0},
            numVertices = {"float", 0.0},
            verticalDistanceIncrement = {"float", 0.0},    
            distancesTexture = {"texture2D"},
            showDebugColors = {"bool", false}
        },
        pass =
        {
            base = "Surface",
            vertex =
            [[
            uniform float time;
            uniform float numVertices; 
            uniform float verticalDistanceIncrement;
            uniform sampler2D distancesTexture;
            uniform bool showDebugColors;
            
            
            // Helper function to decode a float value from RGBA encoding
            float decodeFloatFromRGBA(vec4 encodedColor) {
            
            // Extract sign and decimal shift from alpha channel
            int signIndicator = int(encodedColor.a) / 100; // Extracting the sign: 1 for positive, 2 for negative
            int decimalShift = int(encodedColor.a) % 100; // Extracting the decimal shift
            
            // Reconstruct the original float value        
            float value = (encodedColor.r * 255.0) + (encodedColor.g) + (encodedColor.b / 255.0); 
            
            // Apply the decimal shift
            value *= pow(10.0, float(decimalShift));
            
            // Apply the sign
            if (signIndicator == 2) {
            value = -value;
            }
            return value;
            }
            
            
            void vertex(inout Vertex v, out Input o) {
            // Calculate the index based on the uv.x value
            float index = v.uv.x * 10000.0; // Adjust based on total number of indices
            index = index - mod(index, 6.0); // 
            
            // Fetch the encoded normalized value from the texture
            vec4 encodedColor = texture(distancesTexture, vec2(index / numVertices, 0.5));
            float distance = decodeFloatFromRGBA(encodedColor);
            
            // Apply the undulation effect
            float undulationHeight = sin(distance + (time * 3.0)) * verticalDistanceIncrement * 0.03; // Adjust the effect as needed
            v.position.z += undulationHeight;   
            
            if (showDebugColors == true) {
            // Apply color to vertex for debugging
            v.color.rgba = vec4(distance * 0.05, 0.0, 0.0, 1.0);
            }
            }
            ]]
            ,       
            surface =
            [[
            uniform sampler2D map;
            uniform vec3 diffuse;
            
            void surface(in Input IN, inout SurfaceOutput o)
            {                
            o.emissive = 1.0;                                
            o.emission = vec3(0.0, 0.0, 0.0);
            o.diffuse = IN.color.xyz;        
            }
            ]]
        }
    }
    craft.shader.add(newShader)
    return craft.material(newShaderName)
end

