function vertsAndColorsBasedOn(squaresPerSide, overallSideLength)
    local verts = vertsBasedOn(squaresPerSide, overallSideLength)
    local colors = randomColorsBySquareBelongedTo(#verts)
    local numericalSequence = numericalSequenceTo(#verts)
    --last value only used by craft version
    return verts, colors, numericalSequence
end

function vertsBasedOn(squaresPerSide, overallSideLength)
    local midSide = squaresPerSide/2 - 0.5       
    local squareSideLength = overallSideLength / squaresPerSide
    local verticesForSquares = {}
    for row = 0, squaresPerSide - 1 do
        local rowY = row - midSide
        local ySqr = rowY * rowY
        for column = 0, squaresPerSide - 1 do
            local columnX = column - midSide
            local x = column * squareSideLength
            local towardsScreenTop = row * squareSideLength
            addVecsForSquare(verticesForSquares, x, towardsScreenTop, 0, squareSideLength, craftFlag)
        end
    end
    return verticesForSquares
end

function addVecsForSquare(destinationTable, x, y, z, squareSideLength, craftFlag)
    quickTableInsert(destinationTable, vec3(x, y, z))
    quickTableInsert(destinationTable, vec3(x + squareSideLength, y, z))
    quickTableInsert(destinationTable, vec3(x, y + squareSideLength, z))
    quickTableInsert(destinationTable, vec3(x, y + squareSideLength, z))
    quickTableInsert(destinationTable, vec3(x + squareSideLength, y, z))
    quickTableInsert(destinationTable, vec3(x + squareSideLength, y + squareSideLength, z))
end

function randomColorsBySquareBelongedTo(sizeOfDesiredTable)
    local colorTable = {}
    for i = 1, sizeOfDesiredTable / 6 do
        local newColor = color(
        math.random(255),
        math.random(255),
        math.random(255))
        for chunk = 1, 6 do
            quickTableInsert(colorTable, newColor)
        end
    end
    return colorTable
end

function distancesBySquareBelongedTo(squaresPerSide, verticalIncrement)
    local midSide = squaresPerSide/2 - 0.5       
    local distanceTable = {}
    for row = 0, squaresPerSide - 1 do
        local ySqr = (row - midSide) ^ 2
        for column = 1, squaresPerSide do
            local columnX = column - midSide
            local thisDistance = math.sqrt(columnX * columnX + ySqr) * verticalIncrement / squaresPerSide
            for bunch = 1, 6 do
                quickTableInsert(distanceTable, thisDistance)
            end
        end
    end
    return distanceTable
end

function numericalSequenceTo(highestNumber)
    local sequenceTable = {}
    for i = 1, highestNumber do
        sequenceTable[i] = i
    end
    return sequenceTable
end
