function testImageEncoding()
    
    function generateRandomFloats(count, min, max)
        local floats = {}
        for i = 1, count do
            table.insert(floats, math.random() * (max - min) + min)
        end
        return floats
    end
    
    local randomFloats = generateRandomFloats(100, -500, 500) -- Example: 100 floats between -500 and 500
    print("made randomFloats")
    function encodeFloatsToImage(floats)
        local img = encodeFloatTableAsImage(floats)
        return img
    end
    
    local encodedImage = encodeFloatsToImage(randomFloats)
    print("made encodedImage")
    function decodeFloatsFromImage(img)
        local decodedFloats = {}
        for x = 1, img.width do
            for y = 1, img.height do
                local r, g, b, a = img:get(x, y)
                local result = decodeFloat(r, g, b, a)
                table.insert(decodedFloats, result)
            end
        end
        return decodedFloats
    end
    
    local decodedFloats = decodeFloatsFromImage(encodedImage)
    print("made decodedFloats")
    function verifyEncoding(randomFloats, decodedFloats)
        for i = 1, #randomFloats do
            if math.abs(randomFloats[i] - decodedFloats[i]) > 0.001 then
                print("Mismatch at index", i, ":", randomFloats[i], "~=", decodedFloats[i])
                return false
            end
        end
        print("All values match successfully!")
        return true
    end
    
    verifyEncoding(randomFloats, decodedFloats)
    print("ran verifyEncoding")
    didTestEncoding = true
end

function testEncodingConsistency()
    local testPassed = true
    for i = 1, 100 do
        local testValue = math.random(100000) + math.random()
        testValue = testValue * (math.random() > 0.5 and -1 or 1)
        local r1, g1, b1 = integerSeparationMethod(adjustFloatToTwoDigitsBeforeDecimalAndEncode(testValue)) -- Adjusting testValue to match scale
        local r2, g2, b2, a = encodeFloat(testValue)
        local decodedSeparationValue = decodeIntegerSeparation(r1,g1,b1)
        local decodedDecimalValue = decodeFloat(r2, g2, b2, a)
        
        if not (r1 == r2 and g1 == g2 and b1 == b2) then
            testPassed = false
            print("Test failed for value: ", testValue)
            print("Integer separation method RGB: ", r1, g1, b1)
            print("Decimal preserving method RGB: ", r2, g2, b2)
            break
        else
            print(testValue)
            print(decodedSeparationValue)
            local tweakedDecimal = adjustFloatToTwoDigitsBeforeDecimalAndEncode(decodedDecimalValue)
            print(decodedDecimalValue)
            print(r1,g1,b1)
            print(r2,g2,b2,a)
            print(".")
        end
    end
    
    if testPassed then
        print("All tests passed, encoding methods are consistent.")
    else
        print("Tests failed, encoding methods are not consistent.")
    end
end


function integerSeparationMethod(value)
    -- Directly encode the integer part and the first two decimal parts into RGB channels
    local intPart = math.floor(value)
    local firstDecimalPart = math.floor((value - intPart) * 255)
    local secondDecimalPart = math.floor(((value - intPart) * 255 - firstDecimalPart) * 255)
    return intPart, firstDecimalPart, secondDecimalPart
end

function decodeIntegerSeparation(r, g, b)
    -- Decode the original value from RGB channels
    local value = r + g / 255.0 + b / (255.0 * 255.0)
    return value
end

function compareEncodings(value)
    local r1, g1, b1, a1 = encodeFloat( value )
    local decodedValue1 = decodeFloat(r1, g1, b1, a1)
    
    local r2, g2, b2 = integerSeparationMethod(value)
    local decodedValue2 = decodeIntegerSeparation(r2, g2, b2)
    
    print("Original Value:", value)
    print("Decoded via decodeFloat:", decodedValue1, "RGB:", r1, g1, b1)
    print("Decoded via integer separation:", decodedValue2, "RGB:", r2, g2, b2)
    print("Difference between original and decodeFloat:", math.abs(value - decodedValue1))
    print("Difference between original and integer separation:", math.abs(value - decodedValue2))
    print("-----")
    
    testEncodingConsistency()
end

function adjustFloatToTwoDigitsBeforeDecimalAndEncode(floatValue)
    local adjustmentFactor = 0
    local signIndicator = floatValue >= 0 and 100 or 200 -- 100 for positive, 200 for negative
    
    -- Ensure the value is positive for simplicity in handling
    floatValue = math.abs(floatValue)
    
    -- Adjust the float to have at most two digits before the decimal
    while floatValue >= 100 do
        floatValue = floatValue / 10
        adjustmentFactor = adjustmentFactor + 1
    end
    while floatValue < 10 and adjustmentFactor > 0 do
        floatValue = floatValue * 10
        adjustmentFactor = adjustmentFactor - 1
    end
    
    return floatValue, signIndicator + adjustmentFactor
end

function decodeFloat(r, g, b, a)
    local sign = a >= 200 and -1 or 1
    local adjustmentFactor = a % 100
    
    -- Reconstruct the float from the encoded parts
    -- works in lua
    local value = (r + g / 255.0 + b / (255.0 * 255.0)) * sign
    value = ((r * 255.0) + (g) + (b / 255.0)) * sign; 
    value = value * (10 ^ adjustmentFactor)
    --does not work in shader: float value = encodedColor.r + (encodedColor.g / 255.0) + (encodedColor.b / (255.0 * 255.0));
    --also does not work in shader: float value = (encodedColor.r * 255.0) + (encodedColor.g) + (encodedColor.b / 255.0); 
    --does work in shader: float value = (encodedColor.r * 255.0) + (encodedColor.g) + (encodedColor.b / 255.0); 
    return value
end